package org.jcp.xml.dsig.internal;

import com.sun.org.slf4j.internal.LoggerFactory;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.security.MessageDigest;
import com.sun.org.apache.xml.internal.security.utils.UnsyncByteArrayOutputStream;
import com.sun.org.slf4j.internal.Logger;
import java.io.OutputStream;

public class DigesterOutputStream extends OutputStream
{
    private static final Logger LOG;
    private final boolean buffer;
    private UnsyncByteArrayOutputStream bos;
    private final MessageDigest md;
    
    public DigesterOutputStream(MessageDigest messageDigest) {
        this(messageDigest, false);
    }
    
    public DigesterOutputStream(MessageDigest md, boolean buffer) {
        this.md = md;
        this.buffer = buffer;
        if (buffer) {
            this.bos = new UnsyncByteArrayOutputStream();
        }
    }
    
    @Override
    public void write(int arg0) {
        if (this.buffer) {
            this.bos.write(arg0);
        }
        this.md.update((byte)arg0);
    }
    
    @Override
    public void write(byte[] array, int i, int len) {
        if (this.buffer) {
            this.bos.write(array, i, len);
        }
        if (DigesterOutputStream.LOG.isDebugEnabled()) {
            DigesterOutputStream.LOG.debug("Pre-digested input:");
            StringBuilder sb = new StringBuilder(len);
            for (; i < i + len; ++i) {
                sb.append((char)array[i]);
            }
            DigesterOutputStream.LOG.debug(sb.toString());
        }
        this.md.update(array, i, len);
    }
    
    public byte[] getDigestValue() {
        return this.md.digest();
    }
    
    public InputStream getInputStream() {
        if (this.buffer) {
            return new ByteArrayInputStream(this.bos.toByteArray());
        }
        return null;
    }
    
    @Override
    public void close() throws IOException {
        if (this.buffer) {
            this.bos.close();
        }
    }
    
    static {
        LOG = LoggerFactory.getLogger(DigesterOutputStream.class);
    }
}
